package
{
	import com.coreyoneil.collision.CollisionList;
	
	import flash.display.DisplayObject;
	import flash.display.Sprite;
	import flash.events.Event;

	public class Bullet extends Sprite
	{
	[Embed(source='assets/animations.swf', symbol='Laser')]
	private var bullet_animation:Class;
	
	
	public var vX:Number;
	public var vY:Number;
	
	public var gravityY:Number = 0;
	public var gravityX:Number = 0;
	
	//friction divider: 1 for no friction 0 for max
	public var frictionY:Number = 1;
	public var frictionX:Number = 1;

	public var speed:Number = 2;
	public var angle:Number = 0; //degrees
	
	//margin for self destruction
	public var marginTop:Number;
	public var marginBottom:Number;
	public var marginLeft:Number;
	public var marginRight:Number;
	
	//reference to the target list
	public var tList:CollisionList;
	
	public var collisionMode:String = "pointToBox"; 
	/*
	collision modes:
	pointToBox - probably faster
	boxToBox 
	pointToShape 
	shapeToShape - slower
	*/
	
	public var rotateToDirection:Boolean = false;
	
	public var animation:Sprite;
	
	
	public function Bullet()
		{
		super();
		animation = new bullet_animation();
		addChild(animation);
		addEventListener(Event.ADDED_TO_STAGE,init);
		}
		
	public function init(e:Event):void
		{
		
		if(isNaN(marginTop))
			marginTop = 0;
			
		if(isNaN(marginBottom))
			marginBottom = stage.stageHeight;
			
		if(isNaN(marginLeft))
			marginLeft = 0;
			
		if(isNaN(marginRight))
			marginRight = stage.stageWidth;
		
		//calculate vX and vY from angle and speed
		var radians:Number = angle * Math.PI / 180;
		
		vX = Math.cos(radians) * speed;
		vY = Math.sin(radians) * speed;
		
		addEventListener(Event.ENTER_FRAME, routine);
		}
	
	public function routine(e:Event):void
		{
		
		vY += gravityY;
		vX += gravityX;
		
		vX*= frictionX;
		vY*= frictionY;
		
		//move
		x += vX;
		y += vY;
		
		if(rotateToDirection)
			rotation = Math.atan2(vY, vX) * 180 / Math.PI; 
		
		
		var i:Number;
		//collision
		//pointToShape pointToBox boxToBox shapeToShape
		switch(collisionMode)
		{
		///////////////////
		case "pointToShape":
		
		for(i=1; i<tList.objectArray.length; i++)
			{
			if(tList.objectArray[i].hitTestPoint(x, y, true))
				{
				tList.objectArray[i].collide(this);
				this.collide(tList.objectArray[i]);
				}
			}
		break;
		
		/////////////////
		case "pointToBox":
		
		for(i=1; i<tList.objectArray.length; i++)
			{
			if(tList.objectArray[i].hitTestPoint(x, y, false))
				{
				tList.objectArray[i].collide(this);
				this.collide(tList.objectArray[i]);
				}
			}
		break;
		
		/////////////////
		case "boxToBox":
		
		for(i=1; i<tList.objectArray.length; i++)
			{
			if(this.hitTestObject(tList.objectArray[i]))
				{
				tList.objectArray[i].collide(this);
				this.collide(tList.objectArray[i]);
				}
			}
		break;
		
		case "shapeToShape":
		//check collisions
		
		if(this.width == 0 || this.height ==0)
			trace("Error: this kind of collision doesn't work for sprites with 0 height or 0 width");
		
		//set this as target
		tList.swapTarget(this as DisplayObject);
		var coll:Array = tList.checkCollisions();
		
		//collisions happened
		if(coll.length >0)
			{
			for(i=0; i<coll.length; i++)
				{
				//find the reference
				var other:Object;
				
				if(coll[i].object1 == this)
					other = coll[i].object2;
				else
					other = coll[i].object1;
				
				//do something
				other.collide(this);
				collide(other);
				}
			}
		break;
		}//end collision with targets
		
		
		//check margin
		if(x>marginRight+width)
			destroy();	
		
		if(y>marginBottom+height)
			destroy();	
		
		if(x<marginLeft - width)
			destroy();
		
		if(y<marginTop-height)
			destroy();
		}
		
		public function collide(obj:Object):void
		{
		//trace(this + " collides with "+obj);
		destroy();
		}
		
		public function destroy():void
		{
		removeEventListener(Event.ENTER_FRAME, routine);
		if(this.parent != null)
			this.parent.removeChild(this);
		}
	}
}