﻿/*
This is the "ship" class for a generic 2D shooter game
the Player instance is directly controlled by the user
there are different modes for "ship" movements, rotation and shooting
to easily implement the most common shooter features
*/

package
{
	import com.coreyoneil.collision.CollisionList;
	
	import flash.display.DisplayObjectContainer;
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.events.*;
	import flash.ui.*;

	public class Player extends Sprite
	{
		
	//option 1: swf vector asset
	
	//embed swf symbol as class
	[Embed(source='assets/animations.swf', symbol='Ship')]
	private var ship_normal:Class;
	
	//animation will be a MovieClip (sprite with timeline)
	public var animation:MovieClip;
	
	
	
	//option 2: spritesheet
	//embed png file as class
	/*
	[Embed(source="assets/ship.png")]
	public var shipSpriteSheet:Class;
	//animation will be an instance of the custom class BmpMovieClip that mirrors some of the MovieClip features
	public var animation:BmpMovieClip;

	*/
	
	
	//shoot sound
	[Embed(source="assets/laser.mp3")]
    public var laser_file:Class;
	
	public var laserSound:SoundEffect = new SoundEffect(laser_file);


	//velocity: pixels per step
	public var vX:Number = 0;
	public var vY:Number = 0;
	
	public var gravityY:Number = 0;
	public var gravityX:Number = 0;
	
	
	//MOVEMENT MODE: how the player moves
	/*
	"absolute" 	keyboard controls (side scroller shooter)
	"relative" 	keyboard controls (asteroids / car games)
	"trailer" 	mouse trailer
	[everything else] no movement
	*/
	public var movementMode:String = "absolute";
	
	
	
	//vars for "absolute" mode
	
	//enabled axis
	public var keyboardControlsX:Boolean = true;
	public var keyboardControlsY:Boolean = true;
	
	//acceleration
	public var accX:Number = 1;
	public var accY:Number = 1;
	
	//friction - slow the movement if keys are not pressed - 0 for no inertia
	public var frictionX:Number = 0.9;
	public var frictionY:Number = 0.9;
	
	//maximum speed
	public var maxVX:Number = 5;
	public var maxVY:Number = 5;
	
	
	
	//vars for "relative" mode
		
	//thrust angle in case of keyboardControlsAngle
	public var movementAngle:Number = 0;
	
	//angular speed
	public var movementAngleSpeed:Number = 6;
	
	//the thrust in case of keyboardControlsAngle
	public var thrust:Number = 0;
	//thrust acceleration
	public var thrustAcc:Number = .1;
	//max thrust
	public var maxThrust:Number = 2;
	//friction
	public var frictionThrust:Number = 0.6;
	
	
	
	//vars for "trailer" mode
	
	//mouse trailer control
	public var followMouseX:Boolean = true;
	public var followMouseY:Boolean = true;
	
	//mouse trailer acceleration
	public var easingX:Number = 10;
	public var easingY:Number = 10;
	
	
	//margin for the ship
	public var marginTop:Number;
	public var marginBottom:Number;
	public var marginLeft:Number;
	public var marginRight:Number;
	
	//what happens when it reaches the margin?
	/*
	"destroy" - self destruct
	"wrap" - reappears on the other side
	"stop" - solid margins
	[everything else] - nothing
	*/
	public var onMargin:String = "stop";
	
	
	//ROTATION MODE - how the animation sprite rotates
	/*
	"direction" according to the current X Y velocity
	"mouse" according to mouse position
	"angle" according to movementAngle
	"mirrorX" if velocityX is negative face left (default sprite facing right)
	"mirrorY" if velocityY is negative mirror Y
	"mirrorXY" a combination of the previous two
	[everything else] for no rotation
	*/
	public var rotationMode:String = "";

	
	//what triggers the shoot action
	public var shootTrigger:String = "action"; // "mouse", "action" (keyboard action input), [everything else] for no shooting
		
	//SHOOT MODE - defines the bullet angle
	public var shootMode:String = "normal"; 
	/* 
	"normal" - creates the bullet without assigning angle and speed
	"direction" - bullet shot with player's movement direction
	"rotation" - bullet shot with player's animation angle
	"mouse" - uses player/mouse positions angle
	*/
	
	//the point of emission (offset from anchor point)
	public var bulletX:Number = 0;
	public var bulletY:Number = 0;
	
	public var minShootAngle:Number = 0;
	public var maxShootAngle:Number = 360;
	
	
	//keyboar configuration
	public var LEFT:Number = Keyboard.LEFT;
	public var RIGHT:Number = Keyboard.RIGHT;
	public var UP:Number = Keyboard.UP;
	public var DOWN:Number = Keyboard.DOWN;
	
	public var ACTION:Number = Keyboard.SPACE;
	//ASCII codes can also be used
	//http://www.adobe.com/livedocs/flash/9.0/main/wwhelp/wwhimpl/common/html/wwhelp.htm?context=LiveDocs_Parts&file=00001136.html
	
	
	//COLLISION MANAGEMENT
	
	//collision list: list containing the (references of) objects to be checked against the player
	public var cList:CollisionList;
	//target list: list containing the (references of) objects to be checked against the player's bullets
	public var tList:CollisionList;
	
	
	public var collisionMode:String = "shapeToShape";
	/*
	collision modes:
	pointToBox - probably faster
	boxToBox 
	pointToShape 
	shapeToShape - slower
	*/
	
	
	//the object that contains all the bullets (default: this class parent)
	public var bulletLayer:DisplayObjectContainer;
	
	//flag enabling the shooting action
	public var readyToShoot:Boolean = true;
	
	
	
	
		public function Player()
		{
		super();
		
		//option 1: vector animation from swf embedded above
		animation = new ship_normal();
		
		//option 2: bitmap animation using bitmap embedded above
		//parameters spriteSheet class, sheet width, sheet height, tile width (single frame) 
		/*
		animation = new BmpMovieClip(shipSpriteSheet, 64, 11, 8);
		//for sprites that rotate it's good practice to offset the animation to the center
		//animation.x = -11/2;
		animation.y = -8/2;
		*/
		addChild(animation);
		
		
		//wait for the object to be on the stage
		addEventListener(Event.ADDED_TO_STAGE,init);
		}
		
		
		//the object is on stage
		public function init(e:Event):void
		{
		if(bulletLayer == null)
			bulletLayer = this.parent;
		
		if(isNaN(marginTop))
			marginTop = 0;
			
		if(isNaN(marginBottom))
			marginBottom = stage.stageHeight;
			
		if(isNaN(marginLeft))
			marginLeft = 0;
			
		if(isNaN(marginRight))
			marginRight = stage.stageWidth;
		
		
		addEventListener(Event.ENTER_FRAME, routine);
		
		
		}
		
		
		
		//main routine
		public function routine(e:Event):void
		{
		
		///MOVEMENT
		
		switch(movementMode)
		{
		
		case "absolute":
		
		//X axis
		if(keyboardControlsX)
			{
			//controls
			if(Input.isDown(LEFT))
				vX-= accX;
					
			if(Input.isDown(RIGHT))
				vX+= accX;
				
			//friction
			if(!Input.isDown(LEFT) && !Input.isDown(RIGHT))
				vX*=frictionX;
				
			//limit speed
			if(vX<-maxVX)
				vX = -maxVX;
					
			if(vX>maxVX)
				vX = maxVX;
				
			}
			
		//Y axis
		if(keyboardControlsY)
			{
			//player control
			if(Input.isDown(UP) && vY-accY > -maxVY)
				vY-= accY;
				
			if(Input.isDown(DOWN) && vY+accY < maxVY)
				vY+= accY;
			
			//friction
			if(!Input.isDown(UP) && !Input.isDown(DOWN))
				vY*=frictionY;
			
			
			}
		
		movementAngle = Math.atan2(vY, vX) / Math.PI * 180;
		break;
		
		
		case "relative":
		
		//angle
		if(Input.isDown(LEFT))
			movementAngle-=movementAngleSpeed;
				
		if(Input.isDown(RIGHT))
			movementAngle+=movementAngleSpeed;
			
		//thrust
		if(Input.isDown(UP))
			thrust += thrustAcc;
				
		//braking
		if(Input.isDown(DOWN))
			{
			thrust -= thrustAcc;
			if(thrust<0)
				thrust = 0;
			}
			
		//limit thrust
		if(thrust>maxThrust)
			thrust = maxThrust;
			
		//friction
		if(!Input.isDown(UP))
			thrust*=frictionThrust;
		
		var radians:Number = movementAngle * Math.PI / 180
		
		
		vX += Math.cos(radians) * thrust;
		vY += Math.sin(radians) * thrust;
		
		
		if(vX<-maxVX)
			vX = -maxVX;
					
		if(vX>maxVX)
			vX = maxVX;
		
		if(vY<-maxVY)
			vY = -maxVY;
					
		if(vY>maxVY)
			vY = maxVY;
		
		
		
		break;
		
	
		case "trailer":	
		
		//Mouse trailer (overrides the previous)		
		if(followMouseX)
			vX = mouseX/easingX;
		
		if(followMouseY)
			vY = mouseY/easingY;
		
		break;
		}
		
		
		
		//SHOOT
		switch(shootTrigger)
		{
		case "mouse":
		if(readyToShoot && Input.mouseIsDown)
			{
			shoot();
			readyToShoot = false;
			}
		
		if(!readyToShoot && !Input.mouseIsDown)
			readyToShoot = true;

		break;
		
		case "action":
		//keyboard
		if(readyToShoot && Input.isDown(ACTION))
			{
			shoot();
			readyToShoot = false;
			}
		
		if(!readyToShoot && !Input.isDown(ACTION))
			readyToShoot = true;
		break;
		
		}
		
		
		
		//ROTATION MODE
		
		switch(rotationMode)
		{
		
		case "direction":
		//animation rotation
		if(Math.sqrt(vX*vX + vY*vY)>0.5 )
			animation.rotation = Math.atan2(vY, vX) / Math.PI * 180;
		break;
		
		case "mouse":
		if(Math.sqrt(mouseX*mouseX + mouseY*mouseY)>0.5 )
			animation.rotation = Math.atan2(mouseY, mouseX) / Math.PI * 180;
		break;
		
		case "angle":
		animation.rotation = movementAngle;
		//animation.rotate(20);
		//animation.angle = movementAngle;
		break;
		
		case "mirrorX":
		
		if(vX<0)
			animation.scaleX = -1;
		
		if(vX>0)
			animation.scaleX = 1;
		break;
		
		case "mirrorY":
		
		if(vY<0)
			animation.scaleY = -1;
		
		if(vY>0)
			animation.scaleY = 1;
		break;
		
		
		case "mirrorXY":
		
		if(vX<0)
			animation.scaleX = -1;
		
		if(vX>0)
			animation.scaleX = 1;
			
		if(vY<0)
			animation.scaleY = -1;
		
		if(vY>0)
			animation.scaleY = 1;
		break;
		}
		
		//gravity
		vY += gravityY;
		vX += gravityX;
		
		//update coordinates
		x += vX;
		y += vY;
		
		//MARGINS
		switch(onMargin)
		{
		case "destroy":
		if(x>marginRight || y>marginBottom || x<marginLeft || y<marginTop)
			destroy();
				
		break;
		
		case "wrap":
		if(x>marginRight)
			x = marginLeft;	
		
		if(y>marginBottom)
			y = marginTop;
		
		if(x<marginLeft)
			x = marginRight;
		
		if(y<marginTop)
			y = marginBottom;
			
		break;
		
		case "stop":
		
		if(x>marginRight)
			x = marginRight;	
		
		if(y>marginBottom)
			y = marginBottom;
		
		if(x<marginLeft)
			x = marginLeft;
		
		if(y<marginTop)
			y = marginTop;
		
		break;
		}
		
		var i:Number;
		
		
		
		switch(collisionMode)
		{
		///////////////////
		case "pointToShape":
		
		for(i=1; i<cList.objectArray.length; i++)
			{
			if(cList.objectArray[i].hitTestPoint(x, y, true))
				{
				cList.objectArray[i].collide(this);
				this.collide(cList.objectArray[i]);
				}
			}
		break;
		
		/////////////////
		case "pointToBox":
		
		for(i=1; i<cList.objectArray.length; i++)
			{
			if(cList.objectArray[i].hitTestPoint(x, y, false))
				{
				cList.objectArray[i].collide(this);
				this.collide(cList.objectArray[i]);
				}
			}
		break;
		
		/////////////////
		case "boxToBox":
		
		for(i=1; i<cList.objectArray.length; i++)
			{
			if(this.hitTestObject(cList.objectArray[i]))
				{
				cList.objectArray[i].collide(this);
				this.collide(cList.objectArray[i]);
				}
			}
		break;
		
		case "shapeToShape":
		//check collisions
		
		if(this.width == 0 || this.height ==0)
			trace("Error: this kind of collision doesn't work for sprites with 0 height or 0 width");
		
		var coll:Array = cList.checkCollisions();
		
		//collisions happened
		if(coll.length >0)
			{
			for(i=0; i<coll.length; i++)
				{
				//find the reference
				var other:Object;
				
				if(coll[i].object1 == this)
					other = coll[i].object2;
				else
					other = coll[i].object1;
				
				//do something
				other.collide(this);
				collide(other);
				}
			}
		break;
		}//end collision with targets
				
/*
		
		
		//check collisions
		var coll:Array = cList.checkCollisions();
		
		//collisions happened
		if(coll.length >0)
			{
			for(var i:Number=0; i<coll.length; i++)
				{
				//find the reference
				var other:Object;
				
				if(coll[i].object1 == this)
					other = coll[i].object2;
				else
					other = coll[i].object1;
				
				//do something
				this.collide(other);
				other.collide(this);
				}
			}
		
		}*/
		
		}
		
		/////////////////////////////
		public function shoot():void
		{
		//create bullet
		var newBullet:Bullet = new Bullet();
		newBullet.x = x+bulletX;
		newBullet.y = y+bulletY;
			
		newBullet.speed = 10;
		
		var angle:Number = 0;
		
		//direction depends on the mode
		switch(shootMode)
		{
		case "normal":
		break;
		
		case "mouse":
		angle = Math.atan2(mouseY - bulletY, mouseX - bulletX) * 180 / Math.PI; 
		
		if(angle<minShootAngle)
			angle = minShootAngle;
			
		if(angle>maxShootAngle)
			angle = maxShootAngle;
		
		newBullet.angle = angle; 
		break;
		
		case "rotation":
		angle = degFromRot(animation.rotation);
		
		if(animation is BmpMovieClip)
			angle = animation.angle;
		
		if(angle<minShootAngle)
			angle = minShootAngle;
			
		if(angle>maxShootAngle)
			angle = maxShootAngle;
			 
			
			 
		newBullet.angle = angle; 
		break;
		
		case "direction":
		angle = Math.atan2(vY, vX) * 180 / Math.PI; 
		
		newBullet.angle = angle; 
		break;
		
		
		}
		
		//let the bullet know the targets	
		if(cList!=null)
			newBullet.tList = tList;
		
		bulletLayer.addChild(newBullet);
		
		//make some noise
		laserSound.play();
		}
		
		//utility function that draws a placeholder box
		public function createBox(bx:Number, by:Number, bw:Number, bh:Number):Sprite
		{
		var newBox:Sprite = new Sprite();
		newBox.graphics.lineStyle(3,0x00ff00);
		newBox.graphics.beginFill(0x0000FF);
		newBox.graphics.drawRect(bx, by, bw, bh);
		newBox.graphics.endFill();
		newBox.x = bw/2;
		newBox.y = bh/2;
		return newBox;
		}
		
		public function collide(obj:Object):void
		{
		//trace(this + " collides with "+obj);
		}
		
		//self destruction
		public function destroy():void
		{
		removeEventListener(Event.ENTER_FRAME, routine);
		this.parent.removeChild(this);
		}
		
		public function degFromRot( p_rotInput:Number ):Number
		{
			var degOutput:Number = p_rotInput;
			while ( degOutput >= 360 )
			{
				degOutput -= 360;
			}
			while ( degOutput < 0 )
			{
				degOutput += 360;
			}
			return degOutput;
		}
		
	}
}