package
{
	import com.coreyoneil.collision.CollisionList;
	
	import flash.display.DisplayObjectContainer;
	import flash.display.Sprite;
	import flash.events.*;
	import flash.ui.*;

	public class Player extends Sprite
	{
		
	//mode 1: swf vector file
	//[Embed(source='assets.swf', symbol='Ship')]
	//private var ship_normal:Class;

	//mode 2: spritesheet
	[Embed(source="ship.png")]
	public var shipSpriteSheet:Class;
	
	//public var animation:Sprite;
	public var animation:BmpMovieClip;
	
	//public var animation:MovieClip;
	
	
	//velocity: pixels per step
	public var vX:Number = 0;
	public var vY:Number = 0;
	
	public var gravityY:Number = 0;
	public var gravityX:Number = 0;
	
	//MOVEMENT MODE: how the player moves
	/*
	"absolute" 	keyboard controls (side scroller shooter)
	"relative" 	keyboard controls (asteroids / car games)
	"trailer" 	mouse trailer
	*/
	public var movementMode:String = "absolute";
	
	
	
	//vars for "absolute" mode
	
	//enabled axis
	public var keyboardControlsX:Boolean = true;
	public var keyboardControlsY:Boolean = true;
	
	//acceleration
	public var accX:Number = 1;
	public var accY:Number = 1;
	
	//friction - slow the movement if keys are not pressed - 0 for no inertia
	public var frictionX:Number = 0.9;
	public var frictionY:Number = 0.9;
	
	//maximum speed
	public var maxVX:Number = 5;
	public var maxVY:Number = 5;
	
	
	
	//vars for "relative" mode
		
	//thrust angle in case of keyboardControlsAngle
	public var movementAngle:Number = 0;
	
	//angular speed
	public var movementAngleSpeed:Number = 6;
	
	//the thrust in case of keyboardControlsAngle
	public var thrust:Number = 0;
	//thrust acceleration
	public var thrustAcc:Number = .1;
	//max thrust
	public var maxThrust:Number = 2;
	//friction
	public var frictionThrust:Number = 0.6;
	
	
	
	//vars for "trailer" mode
	
	//mouse trailer control
	public var followMouseX:Boolean = true;
	public var followMouseY:Boolean = true;
	
	//mouse trailer acceleration
	public var easingX:Number = 10;
	public var easingY:Number = 10;
	
	
	
	
	//ROTATION MODE - how the animation sprite rotates
	/*
	"direction" according to the current X Y velocity
	"mouse" according to mouse position
	"angle" according to movementAngle
	"mirrorX" if velocityX is negative face left (default sprite facing right)
	"mirrorY" if velocityY is negative mirror Y
	"mirrorXY" a combination of the previous two
	[everything else] for no rotation
	*/
	public var rotationMode:String = "mirrorX";

	
	//what triggers the shoot action
	public var shootTrigger:String = "action"; // "mouse", "action" (keyboard action input)
		
	//SHOOT MODE - defines the bullet angle
	public var shootMode:String = "normal"; 
	/* 
	"normal" - creates the bullet without assigning angle and speed
	"direction" - bullet shot with player's movement direction
	"rotation" - bullet shot with player's animation angle
	"mouse" - uses player/mouse positions angle
	*/
	
	//key config
	public var LEFT:Number = Keyboard.LEFT;
	public var RIGHT:Number = Keyboard.RIGHT;
	public var UP:Number = Keyboard.UP;
	public var DOWN:Number = Keyboard.DOWN;
	
	public var ACTION:Number = Keyboard.SPACE;
	
	
	
	
	
	
	//list of object to be checked against the player
	public var cList:CollisionList;
	//list of object to be checked against the player's bullets
	public var tList:CollisionList;
	
	public var bulletLayer:DisplayObjectContainer;
	
	public var readyToShoot:Boolean = true;
	
	
	private var lastShootAngle:Number = 0;
	
	
	
		public function Player()
		{
		super();
		
		//mode 1: swf vector file
		//animation = new ship_normal();
		
		//mode 2: bitmap animation using custom class
		animation = new BmpMovieClip(shipSpriteSheet, 64, 11, 8);
	
		addChild(animation);
		
		x = 100;
		y = 100;
		
		addEventListener(Event.ADDED_TO_STAGE,init);
		}
		
		public function init(e:Event):void
		{
		//trace(stage);      
		if(bulletLayer == null)
			bulletLayer = this.parent;
		
		
		addEventListener(Event.ENTER_FRAME, loop);
		}
		
		
		
		//////////////////////////////////
		public function loop(e:Event):void
		{
		
		///MOVEMENT
		
		switch(movementMode)
		{
		
		case "absolute":
		
		//X axis
		if(keyboardControlsX)
			{
			//controls
			if(Input.isDown(LEFT))
				vX-= accX;
					
			if(Input.isDown(RIGHT))
				vX+= accX;
				
			//friction
			if(!Input.isDown(LEFT) && !Input.isDown(RIGHT))
				vX*=frictionX;
				
			//limit speed
			if(vX<-maxVX)
				vX = -maxVX;
					
			if(vX>maxVX)
				vX = maxVX;
				
			}
			
		//Y axis
		if(keyboardControlsY)
			{
			//player control
			if(Input.isDown(UP) && vY-accY > -maxVY)
				vY-= accY;
				
			if(Input.isDown(DOWN) && vY+accY < maxVY)
				vY+= accY;
			
			//friction
			if(!Input.isDown(UP) && !Input.isDown(DOWN))
				vY*=frictionY;
			
			
			}
		
		movementAngle = Math.atan2(vY, vX) / Math.PI * 180;
		break;
		
		
		case "relative":
		
		//angle
		if(Input.isDown(LEFT))
			movementAngle-=movementAngleSpeed;
				
		if(Input.isDown(RIGHT))
			movementAngle+=movementAngleSpeed;
			
		//thrust
		if(Input.isDown(UP))
			thrust += thrustAcc;
				
		//braking
		if(Input.isDown(DOWN))
			{
			thrust -= thrustAcc;
			if(thrust<0)
				thrust = 0;
			}
			
		//limit thrust
		if(thrust>maxThrust)
			thrust = maxThrust;
			
		//friction
		if(!Input.isDown(UP))
			thrust*=frictionThrust;
		
		var radians:Number = movementAngle * Math.PI / 180
		
		
		vX += Math.cos(radians) * thrust;
		vY += Math.sin(radians) * thrust;
		
		
		if(vX<-maxVX)
			vX = -maxVX;
					
		if(vX>maxVX)
			vX = maxVX;
		
		if(vY<-maxVY)
			vY = -maxVY;
					
		if(vY>maxVY)
			vY = maxVY;
		
		
		
		break;
		
	
		case "trailer":	
		
		//Mouse trailer (overrides the previous)		
		if(followMouseX)
			vX = mouseX/easingX;
		
		if(followMouseX)
			vY = mouseY/easingY;
		
		break;
		}
		
		
		
		//SHOOT
		switch(shootTrigger)
		{
		case "mouse":
		if(readyToShoot && Input.mouseIsDown)
			{
			shoot();
			readyToShoot = false;
			}
		
		if(!readyToShoot && !Input.mouseIsDown)
			readyToShoot = true;

		break;
		
		case "action":
		//keyboard
		if(readyToShoot && Input.isDown(ACTION))
			{
			shoot();
			readyToShoot = false;
			}
		
		if(!readyToShoot && !Input.isDown(ACTION))
			readyToShoot = true;
		break;
		
		}
		
		
		
		//ROTATION MODE
		
		switch(rotationMode)
		{
		
		case "direction":
		//animation rotation
		if(Math.sqrt(vX*vX + vY*vY)>0.5 )
			animation.rotation = Math.atan2(vY, vX) / Math.PI * 180;
		break;
		
		case "mouse":
		if(Math.sqrt(mouseX*mouseX + mouseY*mouseY)>0.5 )
			animation.rotation = Math.atan2(mouseY, mouseX) / Math.PI * 180;
		break;
		
		case "angle":
		animation.rotation = movementAngle;
		//animation.rotate(20);
		//animation.angle = movementAngle;
		break;
		
		case "mirrorX":
		
		if(vX<0)
			animation.scaleX = -1;
		
		if(vX>0)
			animation.scaleX = 1;
		break;
		
		case "mirrorY":
		
		if(vY<0)
			animation.scaleY = -1;
		
		if(vY>0)
			animation.scaleY = 1;
		break;
		
		
		case "mirrorXY":
		
		if(vX<0)
			animation.scaleX = -1;
		
		if(vX>0)
			animation.scaleX = 1;
			
		if(vY<0)
			animation.scaleY = -1;
		
		if(vY>0)
			animation.scaleY = 1;
		break;
		}
		
		//gravity
		vY += gravityY;
		vX += gravityX;
		
		//update coordinates
		x += vX;
		y += vY;
		
		//check collisions
		var coll:Array = cList.checkCollisions();
		
		//collisions happened
		if(coll.length >0)
			{
			for(var i:Number=0; i<coll.length; i++)
				{
				//find the reference
				var other:Object;
				
				if(coll[i].object1 == this)
					other = coll[i].object2;
				else
					other = coll[i].object1;
				
				//do something
				other.destroy();
				}
			}
		
		}
		
		
		
		/////////////////////////////
		public function shoot():void
		{
		//create bullet
		var newBullet:Bullet = new Bullet();
		newBullet.x = x;
		newBullet.y = y;
			
		newBullet.speed = 10;
		
		var angle:Number = 0;
		
		//direction depends on the mode
		switch(shootMode)
		{
		case "normal":
		break;
		
		case "mouse":
		angle = Math.atan2(mouseY, mouseX) * 180 / Math.PI; 
		newBullet.angle = angle; 
		break;
		
		case "rotation":
		angle = animation.rotation;
		
		if(animation is BmpMovieClip)
			angle = animation.angle;
			 
		newBullet.angle = angle; 
		break;
		
		case "direction":
		angle = Math.atan2(vY, vX) * 180 / Math.PI; 
		newBullet.angle = angle; 
		break;
		
		
		}
		
			
		if(cList!=null)
			newBullet.tList = tList;
			
		bulletLayer.addChild(newBullet);	
		}
		
		public function destroy():void
		{
		removeEventListener(Event.ENTER_FRAME, loop);
		this.parent.removeChild(this);
		}
		
	}
}