﻿/*
This is a basic implementation of a generic 2D shooter game
*/

package {
	
	//collision detection is based on a package by corey o neil (collision detection kit)
	import com.coreyoneil.collision.CollisionList;
	
	import flash.display.Sprite;
	import flash.events.*;
	import flash.ui.*;
	import flash.utils.*;
	
	//set the size of the stage, the BG color and the framerate
	[SWF(width="800", height="400", backgroundColor="#303030", frameRate="30")]
	
	
	public class game extends Sprite
	{
	//the player controlled ship
	public var hero:Player;
	
	//embedding music file as class
	[Embed(source="assets/musicLoop.mp3")]
	public var sound_file:Class;
	
	//the music is an instance of a custom made class "Loop"
	public var music:Loop;
	
	//these lists are used by the collision detection kit and contain the references of the object that might collide
	//things that might hit the player
	public var collisionList:CollisionList;
	//things that might be hit by player's bullets
	public var targetList:CollisionList;
	
	//workaround to remind the compiler to include the class "thing"
	private var dummy:Thing;
	
	//parameters for the creation of "enemies
	//waves coming from the right border
	public var spawnMinY:Number = 0;
	public var spawnMaxY:Number = 400;
	public var spawnMinX:Number = 800;
	public var spawnMaxX:Number = 800;
	
	public var spawnCheckOverlapping:Boolean = true;
	public var spawnRate:Number = 10; //emit every n frames
	public var spawnCertainty:Number = 50; //0 to 100 - 0 never, 100 always
	
	public var cycle:Number = 0;
	
	//the game starts!
	public function game()
		{
		//initialize input manager (see Input class)
		Input.initialize(stage);
		
		//create an instance of player
		hero = new Player();
		
		//set initial position
		hero.x = 100;
		hero.y = 100;
		
		//put it on stage
		addChild(hero);
		
		//create collision list for hero
		collisionList = new CollisionList(hero);
		hero.cList = collisionList;
		
		//create collision list for hero bullets
		//all the things that can be destroyed
		targetList = new CollisionList(hero);
		hero.tList = targetList;
		
		addEventListener(Event.ENTER_FRAME, routine);
		
		music = new Loop(sound_file, 35, 30);
		
		}
	
		
	//the main routine it's called on every frame
	public function routine(e:Event):void
		{
		//the creation of "enemies"		
		if(cycle%spawnRate==0)
			if(Math.round(Math.random()*100) < spawnCertainty)
				spawnEnemy();

		cycle++;
		}
		
	
	public function spawnEnemy():void
		{
		//create a new "enemy"
		var enemy:Thing = new Thing();
 		
 		//can the enemy collide with the player?
       	collisionList.addItem(enemy);
       	//pass the reference of the list so it can remove himself when it's destroyed
       	enemy.cList = collisionList;
        
       	//is the enemy a target of player's bullets?
        targetList.addItem(enemy);
        //pass the reference of the list so it can remove himself when it's destroyed
        enemy.tList = targetList;
        
     	//randomize positions according to spawn variables
        enemy.x = Math.floor(Math.random() * (spawnMaxX - spawnMinX + 1)) + spawnMinX;
		enemy.y = Math.floor(Math.random() * (spawnMaxY - spawnMinY + 1)) + spawnMinY;
			
        //add to stage
        addChild(enemy);
        
        
        //the following chunk of code prevents the enemy to spawn overlapped by re-randomizing the position
        if(spawnCheckOverlapping)
	        {
	        var over:Boolean = true;
	        var safeCount:Number = 0
	        //rerandomize until not over
	        while(over && safeCount<10)
		        {
		        enemy.x = Math.floor(Math.random() * (spawnMaxX - spawnMinX + 1)) + spawnMinX;
				enemy.y = Math.floor(Math.random() * (spawnMaxY - spawnMinY + 1)) + spawnMinY;
		        
		        //once placed verify overlapping with all the objects in the same container
		        var chk:Boolean = false;
		        for (var i:int=0; i < numChildren && !chk; i++) 
		        {
		        //check with obj of the same class
				if(getChildAt(i) is Thing && !chk && getChildAt(i) != enemy)
					chk = enemy.hitTestObject(getChildAt(i));
				}
				
				over = chk;
				safeCount++;
				}
				
			//if still over probably impossible to NOT overlap, cancel the emission
			if(over)
				{
				enemy.destroy();
				}
			}//end overlapping check
		
		}
	}
}
